@@ -3,8 +3,6 @@
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-from __future__ import print_function
-
 import os
 import sys
 import traceback
@@ -18,8 +16,10 @@ from traitlets import Bool, Unicode
 from .commands import (
     install_extension, uninstall_extension, list_extensions,
     enable_extension, disable_extension, check_extension,
-    link_package, unlink_package, build, get_app_version, HERE
+    link_package, unlink_package, build, get_app_version, HERE,
+    update_extension,
 )
+from .debuglog import DebugLogFileMixin
 
 
 flags = dict(base_flags)
@@ -27,10 +27,6 @@ flags['no-build'] = (
     {'BaseExtensionApp': {'should_build': False}},
     "Defer building the app after the action."
 )
-flags['dev-build'] = (
-    {'BaseExtensionApp': {'dev_build': True}},
-    "Build in Development mode"
-)
 flags['clean'] = (
     {'BaseExtensionApp': {'should_clean': True}},
     "Cleanup intermediate files after the action."
@@ -41,13 +37,28 @@ check_flags['installed'] = (
     {'CheckLabExtensionsApp': {'should_check_installed_only': True}},
     "Check only if the extension is installed."
 )
+
+update_flags = copy(flags)
+update_flags['all'] = (
+    {'UpdateLabExtensionApp': {'all': True}},
+    "Update all extensions"
+)
+
+uninstall_flags = copy(flags)
+uninstall_flags['all'] = (
+    {'UninstallLabExtensionApp': {'all': True}},
+    "Uninstall all extensions"
+)
+
 aliases = dict(base_aliases)
 aliases['app-dir'] = 'BaseExtensionApp.app_dir'
+aliases['dev-build'] = 'BaseExtensionApp.dev_build'
+aliases['debug-log-path'] = 'DebugLogFileMixin.debug_log_path'
 
 VERSION = get_app_version()
 
 
-class BaseExtensionApp(JupyterApp):
+class BaseExtensionApp(JupyterApp, DebugLogFileMixin):
     version = VERSION
     flags = flags
     aliases = aliases
@@ -58,8 +69,8 @@ class BaseExtensionApp(JupyterApp):
     should_build = Bool(True, config=True,
         help="Whether to build the app after the action")
 
-    dev_build = Bool(False, config=True,
-        help="Whether to build in dev mode (defaults to production mode)")
+    dev_build = Bool(True, config=True,
+        help="Whether to build in dev mode (defaults to dev mode)")
 
     should_clean = Bool(False, config=True,
         help="Whether temporary files should be cleaned up after building jupyterlab")
@@ -67,20 +78,12 @@ class BaseExtensionApp(JupyterApp):
     def start(self):
         if self.app_dir and self.app_dir.startswith(HERE):
             raise ValueError('Cannot run lab extension commands in core app')
-        try:
+        with self.debug_logging():
             ans = self.run_task()
             if ans and self.should_build:
                 command = 'build:prod' if not self.dev_build else 'build'
                 build(app_dir=self.app_dir, clean_staging=self.should_clean,
                       logger=self.log, command=command)
-        except Exception as ex:
-            _, _, exc_traceback = sys.exc_info()
-            msg = traceback.format_exception(ex.__class__, ex, exc_traceback)
-            for line in msg:
-                self.log.debug(line)
-            self.log.error('\nErrored, use --debug for full output:')
-            self.log.error(msg[-1].strip())
-            sys.exit(1)
 
     def run_task(self):
         pass
@@ -101,6 +104,25 @@ class InstallLabExtensionApp(BaseExtensionApp):
         ])
 
 
+class UpdateLabExtensionApp(BaseExtensionApp):
+    description = "Update labextension(s)"
+    flags = update_flags
+
+    all = Bool(False, config=True,
+        help="Whether to update all extensions")
+
+    def run_task(self):
+        if not self.all and not self.extra_args:
+            self.log.warn('Specify an extension to update, or use --all to update all extensions')
+            return False
+        if self.all:
+            return update_extension(all_=True, app_dir=self.app_dir, logger=self.log)
+        return any([
+            update_extension(name=arg, app_dir=self.app_dir, logger=self.log)
+            for arg in self.extra_args
+        ])
+
+
 class LinkLabExtensionApp(BaseExtensionApp):
     description = """
     Link local npm packages that are not lab extensions.
@@ -133,11 +155,15 @@ class UnlinkLabExtensionApp(BaseExtensionApp):
 
 class UninstallLabExtensionApp(BaseExtensionApp):
     description = "Uninstall labextension(s) by name"
+    flags = uninstall_flags
+
+    all = Bool(False, config=True,
+        help="Whether to uninstall all extensions")
 
     def run_task(self):
         self.extra_args = self.extra_args or [os.getcwd()]
         return any([
-            uninstall_extension(arg, self.app_dir, logger=self.log)
+            uninstall_extension(arg, all_=self.all, app_dir=self.app_dir, logger=self.log)
             for arg in self.extra_args
         ])
 
@@ -180,7 +206,7 @@ class CheckLabExtensionsApp(BaseExtensionApp):
                 logger=self.log)
             for arg in self.extra_args)
         if not all_enabled:
-            exit(1)
+            self.exit(1)
 
 
 _examples = """
@@ -199,6 +225,7 @@ class LabExtensionApp(JupyterApp):
 
     subcommands = dict(
         install=(InstallLabExtensionApp, "Install labextension(s)"),
+        update=(UpdateLabExtensionApp, "Update labextension(s)"),
         uninstall=(UninstallLabExtensionApp, "Uninstall labextension(s)"),
         list=(ListLabExtensionsApp, "List labextensions"),
         link=(LinkLabExtensionApp, "Link labextension(s)"),
@@ -215,7 +242,7 @@ class LabExtensionApp(JupyterApp):
         # The above should have called a subcommand and raised NoStart; if we
         # get here, it didn't, so we should self.log.info a message.
         subcmds = ", ".join(sorted(self.subcommands))
-        sys.exit("Please supply at least one subcommand: %s" % subcmds)
+        self.exit("Please supply at least one subcommand: %s" % subcmds)
 
 
 main = LabExtensionApp.launch_instance
